"""Integration tests for the Python CCDB bindings."""

from __future__ import annotations

import datetime as dt
import os
from pathlib import Path

import gluex_ccdb
import pytest

TABLE_PATH = "/test/demo/mytable"
FIRST_AVAILABLE = dt.datetime(2013, 2, 22, 19, 40, 35, tzinfo=dt.timezone.utc)


def resolve_db_path() -> Path:
    raw = os.environ.get("CCDB_TEST_SQLITE_CONNECTION", "ccdb.sqlite")
    for candidate in _candidate_paths(raw):
        if candidate.exists():
            return candidate
    pytest.skip(
        "CCDB test database not found. Set CCDB_TEST_SQLITE_CONNECTION or place ccdb.sqlite at the repo root."
    )
    raise FileNotFoundError("CCDB test database not found")


def _candidate_paths(raw: str) -> list[Path]:
    raw_path = Path(raw)
    if raw_path.is_absolute():
        return [raw_path]
    bases = [
        Path(__file__).resolve().parents[2],
        Path(__file__).resolve().parents[4],
    ]
    return [raw_path, *(base / raw_path for base in bases)]


@pytest.fixture(scope="module")
def db() -> gluex_ccdb.CCDB:
    return gluex_ccdb.CCDB(str(resolve_db_path()))


def test_directory_and_table_metadata(db: gluex_ccdb.CCDB):
    root = db.root()
    assert root.full_path() == "/"

    test_dir = db.dir("/test")
    assert test_dir.full_path() == "/test"

    demo_dir = test_dir.dir("demo")
    assert demo_dir.full_path() == "/test/demo"

    table = demo_dir.table("mytable")
    assert table.full_path() == TABLE_PATH

    meta = table.meta
    assert meta.n_rows == 2
    assert meta.n_columns == 3

    columns = table.columns()
    assert [c.name for c in columns] == ["x", "y", "z"]
    assert [c.column_type.name for c in columns] == ["double", "double", "double"]


def test_fetch_across_runs_timestamps_and_variations(db: gluex_ccdb.CCDB):
    table = db.table(TABLE_PATH)

    before_first = db.fetch(
        TABLE_PATH, runs=[0, 1, 2, 3], timestamp="2013-02-22 19:40:34"
    )
    assert before_first == {}

    first = db.fetch(TABLE_PATH, runs=[0, 1, 2, 3], timestamp=FIRST_AVAILABLE)
    assert set(first) == {0, 1, 2, 3}
    for data in first.values():
        assert data.n_rows == 2
        assert data.column_names == ["x", "y", "z"]
        assert data.value("x", 0) == 0.0
        assert data.value("y", 0) == 1.0
        assert data.value("z", 0) == 2.0
        assert data.value("x", 1) == 3.0
        assert data.value("y", 1) == 4.0
        assert data.value("z", 1) == 5.0

    mc = table.fetch(runs=[2], variation="mc", timestamp=FIRST_AVAILABLE)
    assert set(mc) == {2}
    mc_row = mc[2].row(1)
    assert mc_row.value("z") == 5.0

    updated = db.fetch(TABLE_PATH, runs=[0, 1, 2, 3], timestamp="2020-02-01 00:00:00")
    assert set(updated) == {0, 1, 2, 3}
    for data in updated.values():
        assert data.value(0, 0) == 1.0
        assert data.value(1, 0) == 2.0
        assert data.value(2, 0) == 3.0
        row_columns = data.row(1).columns()
        assert [name for name, _, _ in row_columns] == ["x", "y", "z"]
        assert [ctype.name for _, ctype, _ in row_columns] == [
            "double",
            "double",
            "double",
        ]
        assert [value for _, _, value in row_columns] == [4.0, 5.0, 6.0]



from datetime import datetime

class ColumnType:
    @property
    def name(self) -> str: ...

class ColumnMeta:
    @property
    def id(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def column_type(self) -> ColumnType: ...
    @property
    def order(self) -> int: ...
    @property
    def comment(self) -> str: ...

class TypeTableMeta:
    @property
    def id(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def n_rows(self) -> int: ...
    @property
    def n_columns(self) -> int: ...
    @property
    def comment(self) -> str: ...

class Column:
    @property
    def name(self) -> str: ...
    @property
    def column_type(self) -> ColumnType: ...
    def row(self, row: int) -> object: ...
    def values(self) -> list[object]: ...

class RowView:
    @property
    def n_columns(self) -> int: ...
    @property
    def column_types(self) -> list[ColumnType]: ...
    def value(self, column: int | str) -> object | None: ...
    def columns(self) -> list[tuple[str, ColumnType, object]]: ...

class Data:
    @property
    def n_rows(self) -> int: ...
    @property
    def n_columns(self) -> int: ...
    @property
    def column_names(self) -> list[str]: ...
    @property
    def column_types(self) -> list[ColumnType]: ...
    def column(self, column: int | str) -> Column: ...
    def row(self, row: int) -> RowView: ...
    def rows(self) -> list[RowView]: ...
    def value(self, column: int | str, row: int) -> object | None: ...

class TypeTableHandle:
    @property
    def name(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def meta(self) -> TypeTableMeta: ...
    def full_path(self) -> str: ...
    def columns(self) -> list[ColumnMeta]: ...
    def fetch(
        self,
        *,
        runs: list[int] | None = None,
        variation: str | None = None,
        timestamp: str | datetime | None = None,
    ) -> dict[int, Data]: ...
    def fetch_run_period(
        self,
        *,
        run_period: str,
        rest_version: int | None = None,
        variation: str | None = None,
        timestamp: str | datetime | None = None,
    ) -> dict[int, Data]: ...

class DirectoryHandle:
    def full_path(self) -> str: ...
    def parent(self) -> DirectoryHandle | None: ...
    def dirs(self) -> list[DirectoryHandle]: ...
    def dir(self, path: str) -> DirectoryHandle: ...
    def tables(self) -> list[TypeTableHandle]: ...
    def table(self, name: str) -> TypeTableHandle: ...

class CCDB:
    def __init__(self, path: str) -> None: ...
    @property
    def connection_path(self) -> str: ...
    def dir(self, path: str) -> DirectoryHandle: ...
    def table(self, path: str) -> TypeTableHandle: ...
    def root(self) -> DirectoryHandle: ...
    def fetch(
        self,
        path: str,
        *,
        runs: list[int] | None = None,
        variation: str | None = None,
        timestamp: str | datetime | None = None,
    ) -> dict[int, Data]: ...
    def fetch_run_period(
        self,
        path: str,
        *,
        run_period: str,
        rest_version: int | None = None,
        variation: str | None = None,
        timestamp: str | datetime | None = None,
    ) -> dict[int, Data]: ...

__all__ = [
    "CCDB",
    "Column",
    "ColumnMeta",
    "ColumnType",
    "Data",
    "DirectoryHandle",
    "RowView",
    "TypeTableHandle",
    "TypeTableMeta",
]
